module Enumerable exposing (Enumerable, toList)

{-| This module documents a simple approach to enumeration.

    "An enumeration is a complete, ordered listing of all the items in a collection."


# Enumerable

@docs Enumerable, toList

-}


{-| A function that implements an enumeration over a type of value.
-}
type alias Enumerable value =
    value -> value


{-| Get a list of all distinct values enumerated.
The values are ordered according to the enumeration, starting with the given value.
-}
toList : Enumerable value -> value -> List value
toList toNext head =
    let
        toTail current =
            if head == current then
                []

            else
                current :: toTail (toNext current)
    in
    head :: toTail (toNext head)
